@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,24 +16,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import sys, time, os
+import sys, os
 import GNS3.Globals as globals
 import GNS3.Config.Defaults as Defaults
-from GNS3.Utils import translate
-from PyQt4.QtGui import QApplication, QMessageBox
+from PyQt4.QtGui import QApplication
 from PyQt4.QtCore import QVariant, QSettings
 from GNS3.Utils import Singleton
 from GNS3.Workspace import Workspace
-from GNS3.Topology import Topology
-from GNS3.Config.Objects import systemDynamipsConf, systemGeneralConf, systemCaptureConf, systemPemuConf, systemSimhostConf
+from GNS3.Config.Objects import systemDynamipsConf, systemGeneralConf, systemCaptureConf, systemQemuConf
 from GNS3.Globals.Symbols import SYMBOLS, SYMBOL_TYPES
 from GNS3.Config.Config import ConfDB, GNS_Conf
 from GNS3.HypervisorManager import HypervisorManager
-from GNS3.PemuManager import PemuManager
-from GNS3.SimhostManager import SimhostManager
+from GNS3.QemuManager import QemuManager
 from GNS3.Translations import Translator
 from GNS3.DynagenSub import DynagenSub
 from GNS3.ProjectDialog import ProjectDialog
@@ -59,8 +56,7 @@ class Application(QApplication, Singleton):
         self.__topology = None
         self.__dynagen = None
         self.__HypervisorManager = None
-        self.__PemuManager = None
-        self.__SimhostManager = None
+        self.__QemuManager = None
 
         # Dict for storing config
         self.__systconf = {}
@@ -68,8 +64,10 @@ class Application(QApplication, Singleton):
         self.__iosimages = {}
         self.__hypervisors = {}
         self.__libraries = {}
+        self.__qemuimages = {}
         self.iosimages_ids = 0
         self.hypervisors_ids = 0
+        self.qemuimages_ids = 0
 
         # set global app to ourself
         globals.GApp = self
@@ -142,7 +140,7 @@ class Application(QApplication, Singleton):
         """ register the systconf instance
         """
 
-        self.__systconf = sytsconf
+        self.__systconf = systconf
 
     def __getSystConf(self):
         """ return the systconf instance
@@ -166,6 +164,20 @@ class Application(QApplication, Singleton):
 
     iosimages = property(__getIOSImages, __setIOSImages, doc = 'IOS images dictionnary')
     
+    def __setQemuImages(self, qemuimages):
+        """ register the sysconf instance
+        """
+
+        self.__qemuimages = qemuimages
+
+    def __getQemuImages(self):
+        """ return the sysconf instance
+        """
+
+        return self.__qemuimages
+
+    qemuimages = property(__getQemuImages, __setQemuImages, doc = 'Qemu images dictionnary')
+    
     def __setLibraries(self, libraries):
         """ register the sysconf instance
         """
@@ -222,41 +234,25 @@ class Application(QApplication, Singleton):
 
     HypervisorManager = property(__getHypervisorManager, __setHypervisorManager, doc = 'HypervisorManager instance')
 
-    def __setPemuManager(self, PemuManager):
-        """ register the PemuManager instance
+    def __setQemuManager(self, QemuManager):
+        """ register the QemuManager instance
         """
 
-        self.__PemuManager = PemuManager
+        self.__QemuManager = QemuManager
 
-    def __getPemuManager(self):
-        """ return the PemuManager instance
+    def __getQemuManager(self):
+        """ return the QemuManager instance
         """
 
-        return self.__PemuManager
+        return self.__QemuManager
 
-    PemuManager = property(__getPemuManager, __setPemuManager, doc = 'PemuManager instance')
-    
-    def __setSimhostManager(self, SimhostManager):
-        """ register the SimhostManager instance
-        """
-
-        self.__SimhostManager = SimhostManager
-
-    def __getSimhostManager(self):
-        """ return the SimhostManager instance
-        """
-
-        return self.__SimhostManager
-
-    SimhostManager = property(__getSimhostManager, __setSimhostManager, doc = 'SimhostManager instance')
+    QemuManager = property(__getQemuManager, __setQemuManager, doc = 'QemuManager instance')
 
     def run(self, file):
 
         # Instantiation of Dynagen
         self.__dynagen = DynagenSub()
 
-        config_version = int(ConfDB().get('GNS3/version', 0x000402))
-
         self.systconf['dynamips'] = systemDynamipsConf()
         confo = self.systconf['dynamips']
         confo.path = ConfDB().get('Dynamips/hypervisor_path', unicode('/usr/local/bin/dynamips'))
@@ -268,10 +264,14 @@ class Application(QApplication, Singleton):
         confo.ghosting = ConfDB().value("Dynamips/dynamips_ghosting", QVariant(True)).toBool()
         confo.mmap = ConfDB().value("Dynamips/dynamips_mmap", QVariant(True)).toBool()
         confo.sparsemem = ConfDB().value("Dynamips/dynamips_sparsemem", QVariant(False)).toBool()
-        confo.memory_limit =int(ConfDB().get("Dynamips/hypervisor_memory_usage_limit", 512))
+        confo.jitsharing = ConfDB().value("Dynamips/dynamips_jitsharing", QVariant(True)).toBool()
+        if sys.platform.startswith('win'):
+            confo.memory_limit = int(ConfDB().get("Dynamips/hypervisor_memory_usage_limit", 512))
+        else:
+            confo.memory_limit = int(ConfDB().get("Dynamips/hypervisor_memory_usage_limit", 1024))
         confo.udp_incrementation = int(ConfDB().get("Dynamips/hypervisor_udp_incrementation", 100))
         confo.import_use_HypervisorManager = ConfDB().value("Dynamips/hypervisor_manager_import", QVariant(True)).toBool()
-        confo.HypervisorManager_binding = ConfDB().get('Dynamips/hypervisor_manager_binding', unicode('localhost'))
+        confo.HypervisorManager_binding = ConfDB().get('Dynamips/hypervisor_manager_binding', unicode('127.0.0.1'))
 
         # replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
@@ -280,38 +280,70 @@ class Application(QApplication, Singleton):
         confo.path = os.path.expanduser(confo.path)
         confo.workdir = os.path.expanduser(confo.workdir)
 
-        # Pemu config
-        self.systconf['pemu'] = systemPemuConf()
-        confo = self.systconf['pemu']
-        confo.pemuwrapper_path = ConfDB().get('Pemu/pemuwrapper_path', unicode(''))
-        confo.pemuwrapper_workdir = ConfDB().get('Pemu/pemuwrapper_working_directory', unicode(''))
-        confo.external_host = ConfDB().get('Pemu/external_host', unicode(''))
-        confo.enable_PemuManager = ConfDB().value("Pemu/enable_PemuManager", QVariant(True)).toBool()
-        confo.import_use_PemuManager = ConfDB().value("Pemu/pemu_manager_import", QVariant(True)).toBool()
-        confo.default_pix_image = ConfDB().get('Pemu/default_pix_image', unicode(''))
-        confo.default_pix_key = str(ConfDB().get('Pemu/default_pix_key', unicode('0x00000000,0x00000000,0x00000000,0x00000000')))
-        confo.default_pix_serial = str(ConfDB().get('Pemu/default_pix_serial', unicode('0x12345678')))
-        confo.default_base_flash = ConfDB().get('Pemu/default_base_flash', unicode(''))
-        confo.PemuManager_binding = ConfDB().get('Pemu/pemu_manager_binding', unicode('localhost'))
+        # Qemu config
+        self.systconf['qemu'] = systemQemuConf()
+        confo = self.systconf['qemu']
+        confo.qemuwrapper_path = ConfDB().get('Qemu/qemuwrapper_path', unicode(''))
+        confo.qemuwrapper_workdir = ConfDB().get('Qemu/qemuwrapper_working_directory', unicode(''))
+        confo.qemu_path = ConfDB().get('Qemu/qemu_path', unicode('qemu'))
+        confo.qemu_img_path = ConfDB().get('Qemu/qemu_img_path', unicode('qemu-img'))
+        confo.external_hosts = ConfDB().get('Qemu/external_hosts', unicode('localhost:10525')).split(',')
+        confo.enable_QemuManager = ConfDB().value("Qemu/enable_QemuManager", QVariant(True)).toBool()
+        confo.import_use_QemuManager = ConfDB().value("Qemu/qemu_manager_import", QVariant(True)).toBool()
+        confo.QemuManager_binding = ConfDB().get('Qemu/qemu_manager_binding', unicode('127.0.0.1'))
+        confo.qemuwrapper_port = int(ConfDB().get('Qemu/qemuwrapper_port', 10525))
+        confo.qemuwrapper_baseUDP = int(ConfDB().get('Qemu/qemuwrapper_baseUDP', 20000))
+        confo.qemuwrapper_baseConsole = int(ConfDB().get('Qemu/qemuwrapper_baseConsole', 3000))
+        confo.default_pix_image = ConfDB().get('Qemu/default_pix_image', unicode(''))
+        confo.default_pix_memory = int(ConfDB().get('Qemu/default_pix_memory', 128))
+        confo.default_pix_nic = str(ConfDB().get('Qemu/default_pix_nic', unicode('e1000')))
+        confo.default_pix_options = str(ConfDB().get('Qemu/default_pix_options', unicode('-nographic')))
+        confo.default_pix_kqemu = ConfDB().value("Qemu/default_pix_kqemu", QVariant(False)).toBool()
+        confo.default_pix_key = str(ConfDB().get('Qemu/default_pix_key', unicode('0x00000000,0x00000000,0x00000000,0x00000000')))
+        confo.default_pix_serial = str(ConfDB().get('Qemu/default_pix_serial', unicode('0x12345678')))
+        confo.default_junos_image = ConfDB().get('Qemu/default_junos_image', unicode(''))
+        confo.default_junos_memory = int(ConfDB().get('Qemu/default_junos_memory', 96))
+        confo.default_junos_nic = str(ConfDB().get('Qemu/default_junos_nic', unicode('e1000')))
+        confo.default_junos_options = str(ConfDB().get('Qemu/default_junos_options', unicode('')))
+        confo.default_junos_kqemu = ConfDB().value("Qemu/default_junos_kqemu", QVariant(False)).toBool()
+        confo.default_junos_kvm = ConfDB().value("Qemu/default_junos_kvm", QVariant(False)).toBool()
+        confo.default_asa_memory = int(ConfDB().get('Qemu/default_asa_memory', 256))
+        confo.default_asa_nic = str(ConfDB().get('Qemu/default_asa_nic', unicode('e1000')))
+        confo.default_asa_baseUDP = int(ConfDB().get('Qemu/default_asa_baseUDP', 22000))
+        confo.default_asa_baseConsole = int(ConfDB().get('Qemu/default_asa_baseConsole', 7000))
+        confo.default_asa_options = str(ConfDB().get('Qemu/default_asa_options', unicode('-hdachs 980,16,32')))
+        confo.default_asa_kqemu = ConfDB().value("Qemu/default_asa_kqemu", QVariant(False)).toBool()
+        confo.default_asa_kvm = ConfDB().value("Qemu/default_asa_kvm", QVariant(False)).toBool()       
+        confo.default_asa_kernel = ConfDB().get('Qemu/default_asa_kernel', unicode(''))
+        confo.default_asa_initrd = ConfDB().get('Qemu/default_asa_initrd', unicode(''))
+        confo.default_asa_kernel_cmdline = ConfDB().get('Qemu/default_asa_kernel_cmdline', unicode('auto console=ttyS0,9600 bigphysarea=16384 ide1=noprobe'))
+        confo.default_ids_image1 = ConfDB().get('Qemu/default_ids_image1', unicode(''))
+        confo.default_ids_image2 = ConfDB().get('Qemu/default_ids_image2', unicode(''))
+        confo.default_ids_memory = int(ConfDB().get('Qemu/default_ids_memory', 512))
+        confo.default_ids_nic = str(ConfDB().get('Qemu/default_ids_nic', unicode('e1000')))
+        confo.default_ids_options = str(ConfDB().get('Qemu/default_ids_options', unicode("-smbios type=1,product='IDS-4215'")))
+        confo.default_ids_kqemu = ConfDB().value("Qemu/default_ids_kqemu", QVariant(False)).toBool()
+        confo.default_ids_kvm = ConfDB().value("Qemu/default_ids_kvm", QVariant(False)).toBool()
 
         # replace ~user and $HOME by home directory
         if os.environ.has_key("HOME"):
-            confo.pemuwrapper_path = confo.pemuwrapper_path.replace('$HOME', os.environ["HOME"])
-            confo.pemuwrapper_workdir =  confo.pemuwrapper_workdir.replace('$HOME', os.environ["HOME"])
+            confo.qemuwrapper_path = confo.qemuwrapper_path.replace('$HOME', os.environ["HOME"])
+            confo.qemuwrapper_workdir =  confo.qemuwrapper_workdir.replace('$HOME', os.environ["HOME"])
             confo.default_pix_image = confo.default_pix_image.replace('$HOME', os.environ["HOME"])
-            confo.default_base_flash =  confo.default_base_flash.replace('$HOME', os.environ["HOME"])
-        confo.pemuwrapper_path = os.path.expanduser(confo.pemuwrapper_path)
-        confo.pemuwrapper_workdir = os.path.expanduser(confo.pemuwrapper_workdir)
+            confo.default_junos_image = confo.default_junos_image.replace('$HOME', os.environ["HOME"])
+            confo.default_asa_kernel = confo.default_asa_kernel.replace('$HOME', os.environ["HOME"])
+            confo.default_asa_initrd = confo.default_asa_initrd.replace('$HOME', os.environ["HOME"])
+            confo.default_ids_image1 = confo.default_ids_image1.replace('$HOME', os.environ["HOME"])
+            confo.default_ids_image2 = confo.default_ids_image2.replace('$HOME', os.environ["HOME"])
+
+        confo.qemuwrapper_path = os.path.expanduser(confo.qemuwrapper_path)
+        confo.qemuwrapper_workdir = os.path.expanduser(confo.qemuwrapper_workdir)
         confo.default_pix_image = os.path.expanduser(confo.default_pix_image)
-        confo.default_base_flash = os.path.expanduser(confo.default_base_flash)
-        
-        # Simhost config
-        self.systconf['simhost'] = systemSimhostConf()
-        confo = self.systconf['simhost']
-        confo.path = ConfDB().get('Simhost/hypervisor_path', unicode(''))
-        confo.basePort = int(ConfDB().get('Simhost/hypervisor_basePort', 9000))
-        confo.baseUDP = int(ConfDB().get('Simhost/hypervisor_baseUDP', 35000))
-        confo.workdir = ConfDB().get('Simhost/hypervisor_working_directory', unicode(''))
+        confo.default_junos_image = os.path.expanduser(confo.default_junos_image)
+        confo.default_asa_kernel = os.path.expanduser(confo.default_asa_kernel)
+        confo.default_asa_initrd = os.path.expanduser(confo.default_asa_initrd)
+        confo.default_ids_image1 = os.path.expanduser(confo.default_ids_image1)
+        confo.default_ids_image2 = os.path.expanduser(confo.default_ids_image2)
         
         # Capture config
         self.systconf['capture'] = systemCaptureConf()
@@ -331,7 +363,9 @@ class Application(QApplication, Singleton):
         self.systconf['general'] = systemGeneralConf()
         confo = self.systconf['general']
         confo.lang = ConfDB().get('GNS3/lang', unicode('en'))
+        confo.slow_start = int(ConfDB().get('GNS3/slow_start', 0))
         confo.project_startup = ConfDB().value("GNS3/project_startup", QVariant(True)).toBool()
+        confo.relative_paths = ConfDB().value("GNS3/relative_paths", QVariant(True)).toBool()
         confo.use_shell = ConfDB().value("GNS3/use_shell", QVariant(True)).toBool()
         confo.term_cmd = ConfDB().get('GNS3/console', unicode(''))
         confo.project_path = ConfDB().get('GNS3/project_directory', unicode(''))
@@ -359,15 +393,12 @@ class Application(QApplication, Singleton):
         if globals.GApp.systconf['dynamips'].path:
             self.__HypervisorManager = HypervisorManager()
 
-        # PemuManager
-        self.__PemuManager = PemuManager()
-        
-        # SimhostManager
-        if globals.GApp.systconf['simhost'].path:
-            self.__SimhostManager = SimhostManager()
+        # QemuManager
+        self.__QemuManager = QemuManager()
 
         GNS_Conf().IOS_images()
         GNS_Conf().IOS_hypervisors()
+        GNS_Conf().QEMU_images()
         GNS_Conf().Libraries()
         GNS_Conf().Symbols()
         
@@ -405,25 +436,10 @@ class Application(QApplication, Singleton):
             dialog.raise_()
             dialog.activateWindow()
 
-#        elif globals.recordConfiguration and config_version < VERSION_INTEGER:
-#        
-#            reply = QMessageBox.question(self.mainWindow, translate("Application", "Configuration file"), 
-#                                               translate("Application", "Configuration file is not longer compatible, would you like to reset it? (you will have to restart GNS3)"), 
-#                                            QMessageBox.Yes, QMessageBox.No)
-
-#            if reply == QMessageBox.Yes:
-#                ConfDB().clear()
-#                c = ConfDB()
-#                c.set('GNS3/version', VERSION_INTEGER)
-#                c.sync()
-#                QApplication.quit()
-#                sys.exit(0)
-
         retcode = QApplication.exec_()
 
         self.__HypervisorManager = None
-        self.__PemuManager = None
-        self.__SimhostManager = None
+        self.__QemuManager = None
 
         if globals.recordConfiguration:
             # Save the geometry & state of the GUI
@@ -444,6 +460,8 @@ class Application(QApplication, Singleton):
         confo = self.systconf['general']
         c.set('GNS3/lang', confo.lang)
         c.set('GNS3/project_startup', confo.project_startup)
+        c.set('GNS3/relative_paths', confo.relative_paths)
+        c.set('GNS3/slow_start', confo.slow_start)
         c.set('GNS3/console', confo.term_cmd)
         c.set('GNS3/use_shell', confo.use_shell)
         c.set('GNS3/gui_show_status_points', confo.status_points)
@@ -464,31 +482,55 @@ class Application(QApplication, Singleton):
         c.set('Dynamips/clean_working_directory', confo.clean_workdir)
         c.set('Dynamips/dynamips_ghosting', confo.ghosting)
         c.set('Dynamips/dynamips_sparsemem', confo.sparsemem)
+        c.set('Dynamips/dynamips_jitsharing', confo.jitsharing)
         c.set('Dynamips/dynamips_mmap', confo.mmap)
         c.set('Dynamips/hypervisor_memory_usage_limit', confo.memory_limit)
         c.set('Dynamips/hypervisor_udp_incrementation', confo.udp_incrementation)
         c.set('Dynamips/hypervisor_manager_import', confo.import_use_HypervisorManager)
         c.set('Dynamips/hypervisor_manager_binding', confo.HypervisorManager_binding)
-
-        # Pemu config
-        confo = self.systconf['pemu']
-        c.set('Pemu/pemuwrapper_path', confo.pemuwrapper_path)
-        c.set('Pemu/pemuwrapper_working_directory', confo.pemuwrapper_workdir)
-        c.set('Pemu/external_host', confo.external_host)
-        c.set('Pemu/enable_PemuManager', confo.enable_PemuManager)
-        c.set('Pemu/pemu_manager_import', confo.import_use_PemuManager)
-        c.set('Pemu/default_pix_image', confo.default_pix_image)
-        c.set('Pemu/default_pix_key', confo.default_pix_key)
-        c.set('Pemu/default_pix_serial', confo.default_pix_serial)
-        c.set('Pemu/default_base_flash', confo.default_base_flash)
-        c.set('Pemu/pemu_manager_binding', confo.PemuManager_binding)
-
-        # Simhost config
-        confo = self.systconf['simhost']
-        c.set('Simhost/hypervisor_path', confo.path)
-        c.set('Simhost/hypervisor_basePort', confo.basePort)
-        c.set('Simhost/hypervisor_baseUDP', confo.baseUDP)
-        c.set('Simhost/hypervisor_working_directory', confo.workdir)
+        
+        # Qemu config
+        confo = self.systconf['qemu']
+        c.set('Qemu/qemuwrapper_path', confo.qemuwrapper_path)
+        c.set('Qemu/qemuwrapper_working_directory', confo.qemuwrapper_workdir)
+        c.set('Qemu/qemu_path', confo.qemu_path)
+        c.set('Qemu/qemu_img_path', confo.qemu_img_path)
+        external_hosts = ','.join(confo.external_hosts)
+        c.set('Qemu/external_hosts', external_hosts)
+        c.set('Qemu/enable_QemuManager', confo.enable_QemuManager)
+        c.set('Qemu/qemu_manager_import', confo.import_use_QemuManager)
+        c.set('Qemu/qemu_manager_binding', confo.QemuManager_binding)
+        c.set('Qemu/qemuwrapper_port', confo.qemuwrapper_port)
+        c.set('Qemu/qemuwrapper_baseUDP', confo.qemuwrapper_baseUDP)
+        c.set('Qemu/qemuwrapper_baseConsole', confo.qemuwrapper_baseConsole)
+        c.set('Qemu/default_pix_image', confo.default_pix_image)
+        c.set('Qemu/default_pix_memory', confo.default_pix_memory)
+        c.set('Qemu/default_pix_nic', confo.default_pix_nic)
+        c.set('Qemu/default_pix_options', confo.default_pix_options)
+        c.set('Qemu/default_pix_kqemu', confo.default_pix_kqemu)
+        c.set('Qemu/default_pix_key', confo.default_pix_key)
+        c.set('Qemu/default_pix_serial', confo.default_pix_serial)
+        c.set('Qemu/default_junos_image', confo.default_junos_image)
+        c.set('Qemu/default_junos_memory', confo.default_junos_memory)
+        c.set('Qemu/default_junos_nic', confo.default_junos_nic)
+        c.set('Qemu/default_junos_options', confo.default_junos_options)
+        c.set('Qemu/default_junos_kqemu', confo.default_junos_kqemu)
+        c.set('Qemu/default_junos_kvm', confo.default_junos_kvm)
+        c.set('Qemu/default_asa_memory', confo.default_asa_memory)
+        c.set('Qemu/default_asa_nic', confo.default_asa_nic)
+        c.set('Qemu/default_asa_options', confo.default_asa_options)
+        c.set('Qemu/default_asa_kqemu', confo.default_asa_kqemu)
+        c.set('Qemu/default_asa_kvm', confo.default_asa_kvm)
+        c.set('Qemu/default_asa_kernel', confo.default_asa_kernel)
+        c.set('Qemu/default_asa_initrd', confo.default_asa_initrd)
+        c.set('Qemu/default_asa_kernel_cmdline', confo.default_asa_kernel_cmdline)
+        c.set('Qemu/default_ids_image1', confo.default_ids_image1)
+        c.set('Qemu/default_ids_image2', confo.default_ids_image2)
+        c.set('Qemu/default_ids_memory', confo.default_ids_memory)
+        c.set('Qemu/default_ids_nic', confo.default_ids_nic)
+        c.set('Qemu/default_ids_options', confo.default_ids_options)
+        c.set('Qemu/default_ids_kqemu', confo.default_ids_kqemu)
+        c.set('Qemu/default_ids_kvm', confo.default_ids_kvm)
         
         # Capture settings
         confo = self.systconf['capture']
@@ -496,8 +538,8 @@ class Application(QApplication, Singleton):
         c.set('Capture/capture_reader_cmd', confo.cap_cmd)
         c.set('Capture/auto_start_cmd', confo.auto_start)
 
-        # Clear IOS.hypervisors and IOS.images group
-        c.beginGroup("IOs.images")
+        # Clear IOS.hypervisors, IOS.images and QEMU.images group
+        c.beginGroup("IOS.images")
         c.remove("")
         c.endGroup()
 
@@ -505,6 +547,10 @@ class Application(QApplication, Singleton):
         c.remove("")
         c.endGroup()
         
+        c.beginGroup("QEMU.images")
+        c.remove("")
+        c.endGroup()
+        
         # Clear Symbol.libraries group
         c.beginGroup("Symbol.libraries")
         c.remove("")
@@ -538,6 +584,17 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/base_udp", o.baseUDP)
             c.set(basekey + "/base_console", o.baseConsole)
             
+        # Qemu images
+        for (key, o) in self.__qemuimages.iteritems():
+            basekey = "QEMU.images/" + str(o.id)
+            c.set(basekey + "/name", o.name)
+            c.set(basekey + "/filename", o.filename)
+            c.set(basekey + "/memory", o.memory)
+            c.set(basekey + "/nic", o.nic)
+            c.set(basekey + "/options", o.options)
+            c.set(basekey + "/kqemu", o.kqemu)
+            c.set(basekey + "/kvm", o.kvm)
+            
         # Libraries
         id = 0
         for (key, o) in self.__libraries.iteritems():
