@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,10 +12,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import os, sys, cmd, time, shutil
+import os, sys, cmd, socket
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynagen as Dynagen_Namespace
 import GNS3.Dynagen.dynamips_lib as lib
@@ -25,6 +25,7 @@ from GNS3.Utils import translate
 from GNS3.Dynagen.console import Console as Dynagen_Console, getItems
 from GNS3.External.PyCutExt import PyCutExt
 from GNS3.Node.IOSRouter import IOSRouter
+from __main__ import VERSION
 
 class Console(PyCutExt, Dynagen_Console):
 
@@ -34,7 +35,7 @@ class Console(PyCutExt, Dynagen_Console):
                 "exit", "help", "import", "push", "resume",
                 "shell", "stop", "ver", "confreg",
                 "export", "hist", "list", "py",
-                "save", "show", "suspend", "hypervisors"])
+                "save", "show", "suspend", "hypervisors", "versions"])
 
     def __init__(self, parent):
         """ Initialise the Console widget
@@ -136,6 +137,15 @@ class Console(PyCutExt, Dynagen_Console):
 
         if globals.GApp.HypervisorManager:
             globals.GApp.HypervisorManager.showHypervisors()
+            
+    def do_versions(self, args):
+        """ Show GNS3 + libs versions"""
+        
+        import sip
+        print 'GNS3 version is ' + VERSION
+        print 'Qt version is ' + QtCore.QT_VERSION_STR
+        print 'PyQt version is ' + QtCore.PYQT_VERSION_STR
+        print 'SIP version is ' + sip.SIP_VERSION_STR
 
     def do_start(self, args):
         """start  {/all | router1 [router2] ...}\nstart all or a specific router(s)"""
@@ -273,8 +283,9 @@ Examples:
             if command == 'get' or command == 'show':
                 device = params[0]
                 if command == 'get':
-                    if self.dynagen.devices[device].idlepc != None:
-                        print unicode(translate("Console", "%s already has an idlepc value applied.")) % device
+                    current_idlepc = self.dynagen.devices[device].idlepc
+                    if len(params) < 2 or params[1] != 'force' and current_idlepc != None:
+                        print unicode(translate("Console", "%s already has an idlepc value applied (%s).")) % (device, current_idlepc)
                         return
 
                     print translate("Console", "Please wait while gathering statistics...")
@@ -386,7 +397,7 @@ Examples:
             print Dynagen_Console.do_export.__doc__
             return
 
-         # The last item is the directory (or should be anyway)
+        # The last item is the directory (or should be anyway)
         directory = items.pop().strip('"')
 
         if not os.access(directory, os.F_OK):
