@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,11 +16,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 import sys, os
-import GNS3.Globals as globals
 from GNS3.Defaults.AbstractDefaults import AbstractDefaults
 
 class IOSRouterDefaults(AbstractDefaults):
@@ -33,6 +32,7 @@ class IOSRouterDefaults(AbstractDefaults):
         self.chassis = 'None'
         self.default_image = 'None'
         self.default_ghostios = 'False'
+        self.default_jitsharing = 'False'
         self.default_cnfg = 'None'
         self.default_conf = 'None'
         self.default_confreg = '0x2102'
@@ -91,3 +91,13 @@ class IOSRouterDefaults(AbstractDefaults):
                 del self.config['ghostios']
         else:
             self.config['ghostios'] = bool(ghostios)
+            
+    def set_jitsharing(self, jitsharing):
+        """ Enable or disable JIT blocks sharing feature
+        """
+
+        if self.default_jitsharing == jitsharing:
+            if self.config.has_key('jitsharing'):
+                del self.config['jitsharing']
+        else:
+            self.config['jitsharing'] = bool(jitsharing)
