@@ -3,7 +3,7 @@
 
 """
 confConsole.py
-Copyright (C) 2007  Pavel Skovajsa
+Copyright (C) 2007-2010  Pavel Skovajsa
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@ import re
 from dynamips_lib import DynamipsError, DynamipsWarning, GENERIC_7200_PAS, GENERIC_3600_NMS, GENERIC_1700_NMS, GENERIC_2600_NMS, GENERIC_3700_NMS
 
 # Regex matching slot definitions. Used in pre_cmd() function in confRouterConsole
-SLOT_RE = re.compile('^slot[0-6]', re.IGNORECASE)
+SLOT_RE = re.compile('^slot[0-7]', re.IGNORECASE)
 
 # determine if we are in the debugger
 try:
@@ -59,7 +59,7 @@ def debug(string):
 
 class AbstractConsole(cmd.Cmd):
     """abstract console class, all other console and confconsole classes inherit behavior from it"""
-    
+
     def __init__(self):
         cmd.Cmd.__init__(self)
         # Import the main namespace for use in this module
@@ -176,11 +176,12 @@ class confDefaultsConsole(AbstractConsole):
         self.d = self.dynamips_server.host + ':' + str(self.dynamips_server.port)
         self.config = self.dynagen.defaults_config[self.d]
         self.adaptertuple = ()
-        self.max_slots = 6
+        self.max_slots = 7
         self.min_slot = 0
         self.chassis = 'None'
         self.default_image = 'None'
         self.default_ghostios = 'False'
+        self.default_jitsharing = 'False'
         self.default_cnfg = 'None'
         self.default_conf = 'None'
         self.default_confreg = '0x2102'
@@ -190,6 +191,7 @@ class confDefaultsConsole(AbstractConsole):
         self.default_exec_area = 'None'
         self.default_mmap = True
         self.default_sparsemem = 'False'
+        self.default_ghostsize = None
 
     def do_exit(self, args):
         """Exits from the console"""
@@ -222,7 +224,7 @@ class confDefaultsConsole(AbstractConsole):
 
     def clean_args(self, args):
         """clean the arguments from spaces and stuff"""
-        
+
         #get rid of that '='
         argument = args.strip('=')
         #get rid of starting space
@@ -231,7 +233,7 @@ class confDefaultsConsole(AbstractConsole):
 
     def set_int_option(self, option, args):
         """set integer type option in config"""
-        
+
         if '?' in args or args.strip() == '':
             print getattr(self, 'do_' + option).__doc__
             return
@@ -250,7 +252,7 @@ class confDefaultsConsole(AbstractConsole):
 
     def set_string_option(self, option, args):
         """set string type option in config"""
-        
+
         if '?' in args or args.strip() == '':
             print getattr(self, 'do_' + option).__doc__
             return
@@ -375,6 +377,29 @@ class confDefaultsConsole(AbstractConsole):
             error('the only possible options are True or False, not: ' + args)
         self.dynamips_server.configchange = True
 
+    def do_jitsharing(self, args):
+        """jitsharing = {True|False}
+\tenable or disable JIT blocks sharing"""
+
+        if '?' in args or args.strip() == '':
+            print self.do_jitsharing.__doc__
+            return
+
+        jitsharing = self.clean_args(args)
+        if jitsharing in ('True', 'False'):
+            if self.default_jitsharing == jitsharing:
+                if self.config.has_key('jitsharing'):
+                    del self.config['jitsharing']
+            else:
+                if self.config.has_key('image'):
+                    self.config['jitsharing'] = bool(jitsharing)
+                else:
+                    error('specify first the IOS image and AFTER that turn on JIT sharing')
+        else:
+            #implement JIT sharing and port all other instances into it
+            error('the only possible options are True or False, not: ' + args)
+        self.dynamips_server.configchange = True
+
     def do_idlepc(self, args):
         """idlepc = <idlepc_value>
 \tSet the Idle PC value"""
@@ -387,6 +412,13 @@ class confDefaultsConsole(AbstractConsole):
 
         self.set_int_option('idlemax', args)
 
+    def do_ghostsize(self, args):
+        """ghostsize = <number>
+\tsets the size of the ram allocated to the ghost IOS image
+Must be at least as large as the device with the most ram that
+will use this ghost image"""
+
+        self.set_int_option('ghostsize', args)
     def do_idlesleep(self, args):
         """idlesleep = <number>
 \tadvanced manipulation of idlepc"""
@@ -423,7 +455,7 @@ class confDefaultsConsole(AbstractConsole):
                 return
         else:
             error('incorect syntax: ' + args)
-        
+
         self.dynamips_server.configchange = True
 
 
@@ -613,7 +645,7 @@ class conf7200DefaultsConsole(confDefaultsConsole):
             'npe-g1',
             'npe-g2',
             )
-        self.max_slots = 6
+        self.max_slots = 7
         self.chassis = '7200'
         if '7200' in self.config:
             self.config = self.config['7200']
@@ -660,7 +692,7 @@ class conf7200DefaultsConsole(confDefaultsConsole):
                 self.config['npe'] = npe
         else:
             error('this NPE type does not exist: ' + args + '\nProper npe types are: ' + str(self.npeTuple))
-        
+
         self.dynamips_server.configchange = True
 
     def do_midplane(self, args):
@@ -799,7 +831,7 @@ class confRouterConsole(confDefaultsConsole):
             err = e[0]
             error('Connecting %s %s to %s resulted in:    %s' % (self.router.name, source, destination, err))
             return
-        
+
         if result == False:
             error('Attempt to connect %s %s to unknown device: "%s"' % (self.router.name, source, destination))
 
@@ -817,7 +849,7 @@ class confRouterConsole(confDefaultsConsole):
         if adapter.can_be_removed():
             adapter.remove()
             self.router.slot[adapter.slot] = None
-            
+
         self.dynagen.update_running_config()
 
     def do_no(self, args):
@@ -902,6 +934,13 @@ no <option> = <option_value>
                     else:
                         error('Bad ' + lside + 'value: ' + rside)
                     return
+                if lside == 'jitsharing':
+                    if rside in ['True', 'False']:
+                        if bool(rside) == jitsharing:
+                            self.onecmd('jitsharing = ' + str(not jitsharing))
+                    else:
+                        error('Bad ' + lside + 'value: ' + rside)
+                    return
                 if str(getattr(self.router, lside)) == rside:
                     #emit the command lside = default_option, this will effective make the command not visible in config
                     if self.defaults_config.has_key(lside):
@@ -989,6 +1028,24 @@ no <option> = <option_value>
         else:
             error('the only possible options are True or False, not: ' + args)
 
+    def do_jitsharing(self, args):
+        """jitsharing = {True|False}
+\tEnable or disable JIT blocks sharing"""
+
+        if '?' in args or args.strip() == '':
+            print self.do_jitsharing.__doc__
+            return
+
+        jitsharing = self.clean_args(args)
+        if jitsharing in ('True', 'False'):
+            try:
+                self.dynagen.jitshareddevices[self.router.name] = jitsharing
+                self.dynagen.jitsharing()
+            except DynamipsError, e:
+                error(e)
+        else:
+            error('the only possible options are True or False, not: ' + args)
+            
     def do_slot(self, args):
         if '?' in args or args.strip() == '':
             print self.do_slot.__doc__
@@ -1121,10 +1178,10 @@ class conf1700RouterConsole(confRouterConsole):
 
 class AbstractConfSWConsole(AbstractConsole):
     """abstract console for all dynamips emulated switches, that implement the common behaviour"""
-       
+
     def __init__(self):
         AbstractConsole.__init__(self)
-        
+
     def do_1(self, args):
         self.connect('1' + args)
 
@@ -1151,7 +1208,7 @@ class AbstractConfSWConsole(AbstractConsole):
 
     def do_9(self, args):
         self.connect('9' + args)
-    
+
     def precmd(self, line):
         ''' This method is called after the line has been input but before
                 it has been interpreted. If you want to modifdy the input line
@@ -1197,7 +1254,7 @@ class confFRSWConsole(AbstractConfSWConsole):
                     else:
                         self.frsw.map(port1, dlci1, port2, dlci2)
                         self.frsw.map(port2, dlci2, port1, dlci1)
-                        self.dynagen.update_running_config()                    
+                        self.dynagen.update_running_config()
                 except AttributeError:
                     error('semantic error in: ' + args)
                     return
@@ -1227,7 +1284,7 @@ class confFRSWConsole(AbstractConfSWConsole):
                         self.frsw.unmap(port1, dlci1, port2, dlci2)
                         self.frsw.unmap(port2, dlci2, port1, dlci1)
                         self.dynagen.update_running_config()
-                
+
                     else:
                         error('semantic error in: ' + args + ' this connection does not exist')
                 except AttributeError:
@@ -1262,6 +1319,7 @@ class confETHSWConsole(AbstractConfSWConsole):
             error('semantic error in: ' + args + ' , this connection already exists')
             return
         self.dynagen.ethsw_map(self.ethsw, params[0].strip(), params[1].strip())
+        self.dynagen.update_running_config()
 
     def do_no(self, args):
         params = args.split('=')
@@ -1269,10 +1327,11 @@ class confETHSWConsole(AbstractConfSWConsole):
         right_side = params[1].strip()
         try:
             if self.dynagen.running_config[self.d][self.e][left_side] == right_side:
-                self.ethsw.unset_port(int(left_side))
+                self.dynagen.disconnect(self.ethsw, left_side, right_side)
+                self.dynagen.update_running_config()
             else:
                 error('this mapping does not exist')
-        except IndexError:
+        except (IndexError, KeyError):
             error('this mapping does not exist')
         except DynamipsError, e:
             error(e)
@@ -1296,7 +1355,7 @@ class confATMBRConsole(AbstractConfSWConsole):
             right_side = params[1].split(':')
             if len(right_side) == 3:
                 try:
-                    port1 = int(left_side)     
+                    port1 = int(left_side)
                     port2 = int(right_side[0])
                     vpi2 = int(right_side[1])
                     vci2 = int(right_side[2])
@@ -1305,7 +1364,7 @@ class confATMBRConsole(AbstractConfSWConsole):
                         error('semantic error in: ' + args + ' this connection already exists')
                     else:
                         self.atmbr.configure(port1, port2, vpi2, vci2)
-                        self.dynagen.update_running_config()                    
+                        self.dynagen.update_running_config()
                 except AttributeError:
                     error('semantic error in: ' + args)
                     return
@@ -1320,24 +1379,27 @@ class confATMBRConsole(AbstractConfSWConsole):
     def do_no(self, args):
         params = args.split('=')
         if len(params) == 2:
-            left_side = params[0]
-            right_side = params[1].split(':')
+            left_side = params[0].strip()
+            right_side = params[1].strip().split(':')
             if len(right_side) == 3:
                 try:
-                    port1 = int(left_side)     
-                    port2 = int(right_side[0])
+                    port1 = int(left_side)
+                    port2 = int(right_side[0])                  
                     vpi2 = int(right_side[1])
-                    vci2 = int(right_side[2])
-                    left_side = params[0].strip()
-                    right_side = params[1].strip()
+                    #1 = 2:0:201 R1 a1/0
+                    try:
+                        vci2 = int(right_side[2])
+                    except ValueError:
+                        # must be this format "1 = 2:0:201 R1 a1/0"
+                        vci_split = right_side[2].split(' ')
+                        vci2 = int(vci_split[0])
                     #check whether this connection already exists:
                     if self.dynagen.running_config[self.d][self.a][left_side] == right_side:
                         self.atmbr.unconfigure(port1, port2, vpi2, vci2)
                         self.dynagen.update_running_config()
-                
                     else:
                         error('semantic error in: ' + args + ' this connection does not exist')
-                except AttributeError:
+                except (AttributeError, ValueError):
                     error('semantic error in: ' + args)
                     return
                 except DynamipsError, e:
@@ -1381,8 +1443,8 @@ class confATMSWConsole(AbstractConfSWConsole):
                         self.atmsw.mapvp(port1, vpi1, port2, vpi2)
                         self.atmsw.mapvp(port2, vpi2, port1, vpi1)
                         self.dynagen.update_running_config()
-                   
-                elif len(left_side) == 3 and len(right_side) == 3:                  
+
+                elif len(left_side) == 3 and len(right_side) == 3:
                     port1 = int(left_side[0])
                     vpi1 = int(left_side[1])
                     vci1 = int(left_side[2])
@@ -1395,7 +1457,7 @@ class confATMSWConsole(AbstractConfSWConsole):
                     else:
                         self.atmsw.mapvc(port1, vpi1, vci1, port2, vpi2, vci2)
                         self.atmsw.mapvc(port2, vpi2, vci2, port1, vpi1, vci1)
-                        self.dynagen.update_running_config()  
+                        self.dynagen.update_running_config()
                 else:
                     error('invalid syntax in: ' + args)
             except AttributeError:
@@ -1412,7 +1474,7 @@ class confATMSWConsole(AbstractConfSWConsole):
         if len(params) == 2:
             left_side = params[0].split(':')
             right_side = params[1].split(':')
-            try: 
+            try:
                 if len(left_side) == 2 and len(right_side) == 2:
                     port1 = int(left_side[0])
                     dlci1 = int(left_side[1])
@@ -1425,10 +1487,10 @@ class confATMSWConsole(AbstractConfSWConsole):
                         self.atmsw.unmapvp(port1, dlci1, port2, dlci2)
                         self.atmsw.unmapvp(port2, dlci2, port1, dlci1)
                         self.dynagen.update_running_config()
-                
+
                     else:
                         error('semantic error in: ' + args + ' this connection does not exist')
-                    
+
                 if len(left_side) == 3 and len(right_side) == 3:
                     port1 = int(left_side[0])
                     vpi1 = int(left_side[1])
@@ -1799,7 +1861,13 @@ router <new_router_name>
             if devdefaults[model].has_key('ghostios'):
                 if devdefaults[model]['ghostios']:
                     router.ghost_status = 2
-                    router.ghost_file =  router.formatted_ghost_file()
+                    router.ghost_file = router.formatted_ghost_file()
+
+            #implement JIT blocks sharing when creating the router from configConsole
+            #use devdefaults to find out whether we have jitsharing = True, and simply set the jitsharing_group
+            if devdefaults[model].has_key('jitsharing'):
+                if devdefaults[model]['jitsharing']:
+                    self.dynagen.jitshareddevices [router.name] = True
 
             #add router to frontend
             self.dynagen.devices[params[0]] = router
@@ -1840,7 +1908,7 @@ router <new_router_name>
         for r in self.dynagen.running_config[self.d]:
             router_section = self.dynagen.running_config[self.d][r]
             #if r is no a section, but a string....like workingdir
-            if isinstance(router_section, str) or isinstance(router_section, int):
+            if isinstance(router_section, str) or isinstance(router_section, unicode) or isinstance(router_section, int):
                 continue
             router_section_name = router_section.name
             router_name = router_section_name.split(' ')
@@ -1874,7 +1942,7 @@ router <new_router_name>
         if frsw.dynamips != self.dynamips_server:
             error('this device is on different hypervisor: ' + frsw.dynamips.host + ':' + str(frsw.dynamips.port))
             return
-            
+
         #delete all frsw mappings
         frsw_section = self.dynagen.running_config[self.d]['FRSW ' + frsw.name]
         for scalar in frsw_section.scalars:
@@ -1883,7 +1951,7 @@ router <new_router_name>
             if self.dynagen.running_config[self.d]['FRSW ' + frsw.name].has_key(scalar):
                 nested_cmd.onecmd('no ' + scalar + ' = ' + frsw_section[scalar])
             self.dynagen.update_running_config()
-        
+
         self.__delete_all_connections_to_sw(frsw)
 
         #now delete the frsw from backend
@@ -1896,7 +1964,7 @@ router <new_router_name>
 
     def __delete_all_connections_to_sw(self, device):
         """go through all hypervisors and routers and emit "no" version of all connection command that are refering this device. Used in FRSW, ATMSW disconnection"""
-        
+
         #go through all hypervisors and routers and emit "no" version of all ATMSW subcommands
         for h in self.dynagen.running_config:
             if h == 'debug' or h == 'autostart':
@@ -1919,7 +1987,7 @@ router <new_router_name>
                                 router = self.dynagen.devices[router_section.name[7:]]
                                 nested_cmd = self.routerConsoleInstanceMap[router.model_string](router, self.prompt, self.dynagen)
                                 nested_cmd.onecmd('no ' + scalar + ' = ' + router_section[scalar])
-                                
+
     def no_atmsw(self, params):
         """delete the atmsw and all its connections"""
         #check if the atmsw exists
@@ -1932,7 +2000,7 @@ router <new_router_name>
         if atmsw.dynamips != self.dynamips_server:
             error('this device is on different hypervisor: ' + atmsw.dynamips.host + ':' + str(atmsw.dynamips.port))
             return
-            
+
         #delete all atmsw mappings
         atmsw_section = self.dynagen.running_config[self.d]['ATMSW ' + atmsw.name]
         for scalar in atmsw_section.scalars:
@@ -1940,7 +2008,7 @@ router <new_router_name>
             if self.dynagen.running_config[self.d]['ATMSW ' + atmsw.name].has_key(scalar):
                 nested_cmd.onecmd('no ' + scalar + ' = ' + atmsw_section[scalar])
             self.dynagen.update_running_config()
-        
+
         self.__delete_all_connections_to_sw(atmsw)
 
         #now delete the atmsw from backend
@@ -1950,7 +2018,7 @@ router <new_router_name>
         print 'ATM switch ' + params[1] + ' deleted'
         #update the config
         self.dynagen.update_running_config()
-        
+
     def no_atmbr(self, params):
         """delete the atmbr and all its connections"""
         #check if the atmbr exists
@@ -1963,7 +2031,7 @@ router <new_router_name>
         if atmbr.dynamips != self.dynamips_server:
             error('this device is on different hypervisor: ' + atmbr.dynamips.host + ':' + str(atmbr.dynamips.port))
             return
-            
+
         #delete all atmbr mappings
         atmbr_section = self.dynagen.running_config[self.d]['ATMBR ' + atmbr.name]
         for scalar in atmbr_section.scalars:
@@ -1971,7 +2039,7 @@ router <new_router_name>
             if self.dynagen.running_config[self.d]['ATMBR ' + atmbr.name].has_key(scalar):
                 nested_cmd.onecmd('no ' + scalar + ' = ' + atmbr_section[scalar])
             self.dynagen.update_running_config()
-        
+
         self.__delete_all_connections_to_sw(atmbr)
 
         #now delete the atmbr from backend
@@ -1996,7 +2064,7 @@ router <new_router_name>
             if device.dynamips == self.dynamips_server:
                 if isinstance(device, self.namespace.Router) and device.model_string == params[1]:
                     self.onecmd('no router ' + device.name)
-                    
+
         #delete the defaults section
         del self.dynagen.defaults_config[self.d][params[1]]
         print 'Defaults section ' + params[1] + ' on ' + self.d + ' deleted'
@@ -2084,7 +2152,7 @@ frsw <new frsw_name>
             #and let's jump into the confFRSWConsole
             nested_cmd = confFRSWConsole(frsw, self.prompt, self.dynagen)
             nested_cmd.cmdloop()
-    
+
     def do_atmsw(self, args):
         """atmsw <atmsw name>
 \tgo into ATM switch conf mode
@@ -2115,7 +2183,7 @@ atmsw <new atmsw_name>
             #and let's jump into the confFRSWConsole
             nested_cmd = confATMSWConsole(atmsw, self.prompt, self.dynagen)
             nested_cmd.cmdloop()
-            
+
     def do_atmbr(self, args):
         """atmbr <atmbr name>
 \tgo into ATM Bridge conf mode
@@ -2149,7 +2217,7 @@ atmbr <new atmbr_name>
                 nested_cmd.cmdloop()
             except DynamipsError, e:
                 print 'dynamips error in creating ATMBR bridge, ' + str(e)
-                
+
     def do_ethsw(self, args):
         """ethsw <ethsw name>
 \tgo into Ethernet switch conf mode
@@ -2183,8 +2251,8 @@ ethsw <new ethsw_name>
                 nested_cmd.cmdloop()
             except DynamipsError, e:
                 print 'dynamips error in creating ethsw bridge, ' + str(e)
-                
-                
+
+
 class confConsole(AbstractConsole):
 
     def __init__(self, dynagen, console):
@@ -2257,6 +2325,8 @@ class confConsole(AbstractConsole):
 \tdebug output level. Higher numbers produce increasing levels of verbosity. 0 means none (the default). 1 is the same as the -d command line switch """
 
         result = self.set_option('debug', 'int', args)
+        if result == None:
+            return
         if result[0]:
             self.dynagen.debuglevel = result[1]
             debuglevel = self.dynagen.debuglevel
