@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,11 +16,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import GNS3.Ui.svg_resources_rc
-from GNS3.Utils import translate
 from GNS3.Node.IOSRouter1700 import IOSRouter1700
 from GNS3.Node.IOSRouter2600 import IOSRouter2600
 from GNS3.Node.IOSRouter2691 import IOSRouter2691
@@ -29,13 +27,11 @@ from GNS3.Node.IOSRouter3700 import IOSRouter3700
 from GNS3.Node.IOSRouter7200 import IOSRouter7200
 from GNS3.Node.DecorativeNode import DecorativeNode
 from GNS3.Node.Cloud import Cloud
-from GNS3.Node.Hub import Hub
 from GNS3.Node.FRSW import FRSW
 from GNS3.Node.ETHSW import ETHSW
 from GNS3.Node.ATMSW import ATMSW
 from GNS3.Node.ATMBR import ATMBR
-from GNS3.Node.FW import FW
-from GNS3.Node.SIMHOST import SIMHOST
+from GNS3.Node.AnyEmuDevice import FW, ASA, JunOS, IDS, QemuDevice
 
 SYMBOL_TYPES = {
                 IOSRouter1700: 'Router c1700', 
@@ -43,13 +39,16 @@ SYMBOL_TYPES = {
                 IOSRouter2691: 'Router c2691',
                 IOSRouter3600: 'Router c3600',
                 IOSRouter3700: 'Router c3700',
-                IOSRouter7200: 'Router c7200', 
-#                SIMHOST: 'Host', 
+                IOSRouter7200: 'Router c7200',
                 FW: 'PIX firewall', 
+                ASA: 'ASA firewall',
+                JunOS: 'Juniper router',
+                IDS: 'IDS',
                 ETHSW: 'Ethernet switch',
                 ATMBR: 'ATM bridge',
                 ATMSW: 'ATM switch', 
-                FRSW: 'Frame Relay switch', 
+                FRSW: 'Frame Relay switch',
+                QemuDevice: 'Qemu host',
                 Cloud: 'Cloud', 
                 DecorativeNode: 'Decorative node', 
                 }
@@ -98,6 +97,18 @@ SYMBOLS = [
     'translated': True,
     },
 
+    {'name': "ASA firewall", 'object': ASA,
+    'normal_svg_file': ":/symbols/PIX_firewall.normal.svg",
+    'select_svg_file': ":/symbols/PIX_firewall.selected.svg", 
+    'translated': True,
+    },
+
+    {'name': "Juniper router", 'object': JunOS,
+    'normal_svg_file': ":/symbols/router.normal.svg",
+    'select_svg_file': ":/symbols/router.selected.svg", 
+    'translated': True,
+    },
+
     {'name': "Ethernet switch", 'object': ETHSW,
     'normal_svg_file': ":/symbols/ethernet_switch.normal.svg",
     'select_svg_file': ":/symbols/ethernet_switch.selected.svg", 
@@ -120,13 +131,19 @@ SYMBOLS = [
     'normal_svg_file': ":/symbols/frame_relay_switch.normal.svg",
     'select_svg_file': ":/symbols/frame_relay_switch.selected.svg", 
     'translated': True,
-    } ,
+    },
     
-#    {'name': "Host", 'object': SIMHOST,
-#    'normal_svg_file': ":/symbols/computer.normal.svg",
-#    'select_svg_file': ":/symbols/computer.selected.svg", 
-#    'translated': True,
-#    },
+    {'name': "IDS", 'object': IDS,
+    'normal_svg_file': ":/symbols/ids.normal.svg",
+    'select_svg_file': ":/symbols/ids.selected.svg", 
+    'translated': True,
+    },
+    
+    {'name': "Qemu host", 'object': QemuDevice,
+    'normal_svg_file': ":/symbols/computer.normal.svg",
+    'select_svg_file': ":/symbols/computer.selected.svg", 
+    'translated': True,
+    },
 
     {'name': "Cloud", 'object': Cloud,
     'normal_svg_file': ":/symbols/cloud.normal.svg",
