@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,15 +16,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import time, os
+import time, os, sys
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
-from socket import socket, timeout, AF_INET, SOCK_STREAM
+from socket import socket, AF_INET, SOCK_STREAM
 from PyQt4 import QtCore, QtGui
-from GNS3.Utils import translate, debug
+from GNS3.Utils import translate, debug, killAll
 from GNS3.Node.IOSRouter import IOSRouter
 
 class HypervisorManager(object):
@@ -59,6 +59,7 @@ class HypervisorManager(object):
         """
 
         proc = QtCore.QProcess(globals.GApp.mainWindow)
+        
         if self.hypervisor_wd:
             # set the working directory
             proc.setWorkingDirectory(self.hypervisor_wd)
@@ -69,9 +70,19 @@ class HypervisorManager(object):
         s.settimeout(300)
         try:
             s.connect(('localhost', port))
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',
-                                      unicode(translate("HypervisorManager", "A hypervisor is already running on port %i, it will not be shutdown after you quit GNS3")) % port)
             s.close()
+
+            reply = QtGui.QMessageBox.question(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"), unicode(translate("HypervisorManager", "Apparently an hypervisor is already running on port %i, would you like to kill all Dynamips processes?")) % port,
+                                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+            if reply == QtGui.QMessageBox.Yes:       
+                killAll(os.path.basename(self.hypervisor_path))
+
+            s.connect(('localhost', port))
+            s.close()
+    
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("HypervisorManager", "Hypervisor Manager"),
+                                       unicode(translate("HypervisorManager", "A program is still running on port %i, you will have to stop it manually or change port settings")) % port)
+
             globals.hypervisor_baseport += 1
             return None
         except:
@@ -85,8 +96,8 @@ class HypervisorManager(object):
             return None
 
         hypervisor = {'port': port,
-                            'proc_instance': proc,
-                            'load': 0}
+                      'proc_instance': proc,
+                      'load': 0}
 
         self.hypervisors.append(hypervisor)
         return hypervisor
@@ -135,8 +146,10 @@ class HypervisorManager(object):
         else:
             if last_exception:
                 debug("Hypervisor manager: last exception raised by socket: " + unicode(last_exception))
+            else:
+                last_exception = 'Unknown problem'
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',
-                                       unicode(translate("HypervisorManager", "Can't connect to the hypervisor on port %i")) % hypervisor['port'])
+                                       unicode(translate("HypervisorManager", "Can't connect to the hypervisor on port %i: %s")) % (hypervisor['port'], last_exception))
             hypervisor['proc_instance'].close()
             self.hypervisors.remove(hypervisor)
             return False
@@ -185,7 +198,8 @@ class HypervisorManager(object):
         if not dynamips_hypervisor:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Hypervisor Manager',
                                         unicode(translate("HypervisorManager", "Can't set up hypervisor on port %i, please check the settings (writable working directory ...)")) % hypervisor['port'])
-            del globals.GApp.dynagen.dynamips[globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])]
+            if globals.GApp.dynagen.dynamips.has_key(globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])):
+                del globals.GApp.dynagen.dynamips[globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + str(hypervisor['port'])]
             hypervisor['proc_instance'].close()
             hypervisor['proc_instance'] = None
             count = 0
@@ -216,6 +230,19 @@ class HypervisorManager(object):
                 if hypervisor['load'] <= 0:
                     hypervisor['load'] = 0
                 break
+            
+    def changeHypervisorLoad(self, node, port, old_default_ram):
+        """ Change the hypervisor RAM load for a given node
+        """
+
+        for hypervisor in self.hypervisors:
+            if hypervisor['port'] == int(port):
+                debug("Hypervisor manager: change hypervisor load (port: " + str(port) + ") for node " + node.hostname)
+                hypervisor['load'] -= old_default_ram
+                if hypervisor['load'] <= 0:
+                    hypervisor['load'] = 0
+                hypervisor['load'] += node.default_ram
+                break
 
     def getHypervisor(self, port):
         """ Get a hypervisor from the hypervisor manager
