@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 import os, math, time, sys
@@ -51,7 +51,7 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
         self.fake = Fake
         self.multi = Multi
 
-        self.setZValue(1)
+        self.setZValue(-1)
 
         if not self.fake:
 
@@ -82,10 +82,10 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
             # set item focusable
             self.setFlag(self.ItemIsFocusable)
 
-            self.encapsulationTransform = {'ETH': 'EN10MB',
-                                                            'FR': 'FRELAY',
-                                                            'HDLC': 'C_HDLC',
-                                                            'PPP': 'PPP_SERIAL'}
+            self.encapsulationTransform = { 'ETH': 'EN10MB',
+                                            'FR': 'FRELAY',
+                                            'HDLC': 'C_HDLC',
+                                            'PPP': 'PPP_SERIAL'}
 
         else:
             src_rect = self.source.boundingRect()
@@ -150,6 +150,7 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
 
         self.setToolTip(unicode(translate("AbstractEdge", "Link: %s (%s) -> %s (%s)")) % (self.source.hostname, self.srcIf, self.dest.hostname, self.destIf))
 
+                
     def keyReleaseEvent(self, event):
         """ Key release handler
         """
@@ -172,12 +173,12 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
             if self.fake:
                 return
             menu = QtGui.QMenu()
-            menu.addAction(QtGui.QIcon(':/icons/delete.svg'), translate("AbstractEdge", "Delete"))
             if self.capturing == True:
                 menu.addAction(QtGui.QIcon(':/icons/inspect.svg'), translate("AbstractEdge", "Stop the capture"))
                 menu.addAction(QtGui.QIcon(':/icons/wireshark.png'), translate("AbstractEdge", "Start Wireshark"))
             else:
                 menu.addAction(QtGui.QIcon(':/icons/inspect.svg'), translate("AbstractEdge", "Capture"))
+            menu.addAction(QtGui.QIcon(':/icons/delete.svg'), translate("AbstractEdge", "Delete"))
             menu.connect(menu, QtCore.SIGNAL("triggered(QAction *)"), self.mousePressEvent_actions)
             menu.exec_(QtGui.QCursor.pos())
 
@@ -202,11 +203,10 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
         iftype = interface[0]
         if iftype == 'e' or iftype == 'f' or iftype == 'g':
             options.append(hostname + ' ' +  interface + ' (encapsulation:ETH)')
-        elif iftype == 's' and isinstance(dest, FRSW):
-            options.append(hostname + ' ' +  interface + ' (encapsulation:FR)')
         elif iftype == 's':
             options.append(hostname + ' ' +  interface + ' (encapsulation:HDLC)')
             options.append(hostname + ' ' +  interface + ' (encapsulation:PPP)')
+            options.append(hostname + ' ' +  interface + ' (encapsulation:FR)')
         else:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractEdge", "Capture"),
                                            translate("AbstractEdge", "Packet capture is not supported on this link type"))
@@ -276,7 +276,7 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
                 globals.GApp.dynagen.devices[device].slot[slot].filter(inttype, port,'capture','both', encapsulation + " " + '"' + self.capfile + '"')
                 self.captureInfo = (device, slot, inttype, port)
                 self.capturing = True
-                print "Capturing to " + self.capfile
+                debug("Capturing to " + self.capfile)
             except lib.DynamipsError, msg:
                 QtGui.QMessageBox.critical(self, translate("AbstractEdge", "Dynamips error"),  unicode(msg))
                 return
@@ -323,7 +323,7 @@ class AbstractEdge(QtGui.QGraphicsPathItem, QtCore.QObject):
             path = capture_conf.cap_cmd.replace("%c", '"' + self.capfile + '"')
             debug("Start Wireshark like application: " + path)
             if sys.platform.startswith('win'):
-                 sub.Popen(path)
+                sub.Popen(path)
             else:
                 sub.Popen(path, shell=True)
         except (OSError, IOError), e:
