@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,17 +16,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 from GNS3.Node.AbstractNode import AbstractNode
 from PyQt4 import QtCore, QtGui
-from GNS3.Utils import translate, debug
+from GNS3.Utils import translate
 import GNS3.Globals as globals
 
-decoration_id = 0
+decoration_id = 1
 
-def init_decoration_id(id = 0):
+def init_decoration_id(id = 1):
     global decoration_id
     decoration_id = id
 
@@ -53,7 +53,9 @@ class DecorativeNode(AbstractNode):
         self.setCustomToolTip()
         self.config = None
         
-        self.setZValue(2)
+    def __del__(self):
+    
+        pass
 
     def set_hostname(self, hostname):
         """ Set a hostname
@@ -77,13 +79,22 @@ class DecorativeNode(AbstractNode):
         """
 
         return self.config
+        
+    def duplicate_config(self):
+        """ Returns a copy of the configuration
+        """
+        
+        config = self.config.copy()
+        config['interfaces'] = list(self.config['interfaces'])
+        return (config)
 
     def set_config(self, config):
         """ Set a configuration
             config: dict
         """
-
-        self.config = config
+        
+        self.config = config.copy()
+        self.config['interfaces'] = list(config['interfaces'])
 
     def getInterfaces(self):
         """ Returns all interfaces
