@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,10 +16,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import os, re, shutil, glob
+import os, shutil, glob, sys
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.dynagen as dynagen_namespace
@@ -27,6 +27,7 @@ import GNS3.Telnet as console
 from PyQt4 import QtCore, QtGui,  QtSvg
 from GNS3.Utils import translate, debug, error
 from GNS3.Node.AbstractNode import AbstractNode
+from GNS3.StartupConfigDialog import StartupConfigDialog
 
 MODULES_INTERFACES = {
     "PA-A1": ('a', 1),
@@ -43,9 +44,9 @@ MODULES_INTERFACES = {
 SLOTLESS_MODELS = ('1710', '1720', '1721', '1750')
 
 # base ID for routers
-router_id = 0
+router_id = 1
 
-def init_router_id(id = 0):
+def init_router_id(id = 1):
     global router_id
     router_id = id
 
@@ -199,12 +200,20 @@ class IOSRouter(AbstractNode):
         return self.local_config
 
     def get_config(self):
-        """ Returns the local configuration copy
+        """ Returns the local configuration
         """
 
         assert(self.router and self.local_config)
         return self.local_config
 
+    def duplicate_config(self):
+        """ Returns a copy of the local configuration
+        """
+
+        config = self.local_config.copy()
+        config['slots'] = list(self.local_config['slots'])
+        return config
+        
     def set_config(self, config):
         """ Set a configuration in Dynamips
             config: dict
@@ -226,31 +235,35 @@ class IOSRouter(AbstractNode):
         # configure the slots
         slot_number = 0
         slot_changed = False
-        for module_name in self.local_config['slots']:
+        for module_name in config['slots']:
             if module_name and self.router.slot[slot_number] == None:
                 self.set_slot(slot_number, module_name)
-                slot_changed = True
+                if not self.router.slot[slot_number].can_be_removed():
+                    slot_changed = True
             elif self.router.slot[slot_number] and module_name and module_name != self.router.slot[slot_number].adapter:
                 self.clean_slot(self.router.slot[slot_number])
                 self.set_slot(slot_number, module_name)
-                slot_changed = True
+                if not self.router.slot[slot_number].can_be_removed():
+                    slot_changed = True
             elif module_name == None and self.router.slot[slot_number]:
                 self.clean_slot(self.router.slot[slot_number])
             slot_number += 1
 
-        if slot_changed and self.router.state == "running" and self.router.model != "c7200":
+        if slot_changed and self.router.model != 'c7200' and self.state == 'running':
             QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("IOSRouter", "Slots"), translate("IOSRouter", "You have to restart this router to use new modules"))
 
         # configure wics if available
-        if self.local_config['wics']:
+        if config['wics']:
             wic_number = 0
-            for wic_name in self.local_config['wics']:
+            for wic_name in config['wics']:
                 if wic_name and not self.router.slot[0].wics[wic_number]:
                     # consider that all wics are in slot 0
                     debug('Install ' + wic_name + ' in wic port ' + str(wic_number))
                     self.router.installwic(wic_name, 0, wic_number)
                 wic_number += 1
 
+        self.local_config = config.copy()
+        self.local_config['slots'] = list(config['slots'])
         self.dynagen.update_running_config()
         self.running_config =  self.dynagen.running_config[self.d][self.r]
         debug("Node " + self.hostname + ": running config: " + str(self.running_config))
@@ -282,11 +295,7 @@ class IOSRouter(AbstractNode):
         """ Set a dynagen device in this node, used for .net import
         """
 
-        model = self.model
-        #devdefaults = self.get_devdefaults()
         self.router = router
-        #self.dynagen.setdefaults(router, devdefaults[model])
-        #self.dynagen.update_running_config()
         self.running_config = self.dynagen.running_config[self.d][self.r]
         self.defaults_config = self.dynagen.defaults_config[self.d][self.router.model_string]
         self.create_config()
@@ -294,15 +303,12 @@ class IOSRouter(AbstractNode):
     def changeStartupConfig(self):
         """ Called to change the startup-config
         """
+     
+        startupConfigDlg = StartupConfigDialog(self.router)
+        startupConfigDlg.setWindowTitle(unicode(translate("IOSRouter", "Startup-Config for %s")) % self.hostname)
+        startupConfigDlg.show()
+        startupConfigDlg.exec_()
 
-        (startup_config,  ok) = QtGui.QInputDialog.getText(globals.GApp.mainWindow, translate("IOSRouter", "Startup-config"),
-                                           unicode(translate("IOSRouter", "Startup-config for %s:")) % self.hostname, QtGui.QLineEdit.Normal, unicode(self.router.cnfg))
-        if ok and startup_config and startup_config != 'None':
-            try:
-                self.router.cnfg = unicode(startup_config)
-            except lib.DynamipsError, msg:
-                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "Startup-config"), unicode(msg))
-        
     def smart_interface(self, link_type):
         """ Pick automatically (if possible) the right interface and adapter for the desired link type
             link_type: an one character string 'g', 'f', 'e', 's', 'a', or 'p'
@@ -310,6 +316,7 @@ class IOSRouter(AbstractNode):
         """
 
         connected_interfaces = self.getConnectedInterfaceList()
+        previousConfig = self.duplicate_config()
 
         # remove unused module from the slots
         for module in self.router.slot:
@@ -329,7 +336,21 @@ class IOSRouter(AbstractNode):
                     if found == False:
                         self.clean_slot(module)
 
-        # try to automatically asign a WIC ...
+        # try to find an empty interface in an occupied slot
+        for module in self.router.slot:
+            if module and module.adapter != 'NM-16ESW':
+                interfaces = module.interfaces
+                for interface_type in interfaces.keys():
+                    if interface_type == link_type:
+                        for port in interfaces[link_type].keys():
+                            if self.router.model_string in SLOTLESS_MODELS:
+                                interface_name = interface_type + str(port)
+                            else:
+                                interface_name = interface_type + str(module.slot) + '/' + str(port)
+                            if interface_name not in connected_interfaces:
+                                return (interface_name)
+
+        # try to automatically assign a WIC ...
         if self.local_config['wics']:
             wic_number = 0
             new_wic = None
@@ -347,7 +368,7 @@ class IOSRouter(AbstractNode):
                 debug('Install ' + new_wic + ' in wic port ' + str(wic_number))
                 self.router.installwic(new_wic, 0, wic_number)
 
-        # try to find an empty interface in an occupied slot
+        # try again to find an empty interface in an occupied slot
         for module in self.router.slot:
             if module and module.adapter != 'NM-16ESW':
                 interfaces = module.interfaces
@@ -374,7 +395,8 @@ class IOSRouter(AbstractNode):
                     (int_type, number_of_interfaces) = MODULES_INTERFACES[module_name][0:2]
                     if int_type == link_type:
                         self.local_config['slots'][slot_number] = module_name
-                        self.set_config(self.local_config)
+                        config = self.duplicate_config()
+                        self.setUndoConfig(config, previousConfig)
                         interfaces = self.router.slot[slot_number].interfaces
                         port = interfaces[int_type][0]
                         interface_name = int_type + str(slot_number) + '/' + str(port)
@@ -442,6 +464,12 @@ class IOSRouter(AbstractNode):
         self.router = router
         debug('Router ' + router.name + ' created')
 
+        #implement JIT blocks sharing
+        if devdefaults[model].has_key('jitsharing'):
+            if devdefaults[model]['jitsharing']:
+                if self.dynagen.jitshareddevices.has_key(self.hostname):
+                    globals.GApp.dynagen.jitsharing()
+
         self.dynagen.update_running_config()
         self.running_config = self.dynagen.running_config[self.d][self.r]
         self.defaults_config = self.dynagen.defaults_config[self.d][self.router.model_string]
@@ -554,6 +582,15 @@ class IOSRouter(AbstractNode):
             self.state = self.router.state
             self.updateToolTips()
             globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
+            
+    def reloadNode(self, progress=False):
+        """ Reload this node
+        """
+ 
+        if self.router.state != 'running':
+            return
+        self.stopNode(progress)
+        self.startNode(progress)
 
     def suspendNode(self, progress=False):
         """ Suspend this node
