@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,31 +16,29 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import re
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
+import GNS3.UndoFramework as undo
 from PyQt4 import QtCore, QtGui, QtSvg
 from GNS3.Topology import Topology
 from GNS3.Utils import translate, debug
 from GNS3.Annotation import Annotation
-from GNS3.ShapeItem import AbstractShapeItem
-from GNS3.ShapeItem import Rectangle
-from GNS3.ShapeItem import Ellipse
+from GNS3.ShapeItem import AbstractShapeItem, Rectangle, Ellipse
 from GNS3.StyleDialog import StyleDialog
+from GNS3.MACTableDialog import MACTableDialog
 from GNS3.Pixmap import Pixmap
 from GNS3.NodeConfigurator import NodeConfigurator
 from GNS3.Node.AbstractNode import AbstractNode
 from GNS3.Globals.Symbols import SYMBOLS, SYMBOL_TYPES
 from GNS3.Node.IOSRouter import IOSRouter
-from GNS3.Node.FW import FW
+from GNS3.Node.AnyEmuDevice import AnyEmuDevice
 from GNS3.Node.FRSW import FRSW
-from GNS3.Node.ETHSW import ETHSW
 from GNS3.Node.ATMSW import ATMSW
-from GNS3.Node.Hub import Hub
-from GNS3.Link.AbstractEdge import AbstractEdge
+from GNS3.Node.ETHSW import ETHSW
+from GNS3.Node.ATMBR import ATMBR
 from GNS3.Link.Ethernet import Ethernet
 from GNS3.Link.Serial import Serial
 
@@ -103,7 +101,7 @@ class Scene(QtGui.QGraphicsView):
 
         menu = QtGui.QMenu()
 
-        instances = map(lambda item: not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not  isinstance(item, AbstractShapeItem), items)
+        instances = map(lambda item: not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem), items)
         if True in instances:
 
             # Action: Configure (Configure the node)
@@ -121,12 +119,31 @@ class Scene(QtGui.QGraphicsView):
             showHostnameAct.setIcon(QtGui.QIcon(":/icons/show-hostname.svg"))
             self.connect(showHostnameAct, QtCore.SIGNAL('triggered()'), self.slotShowHostname)
 
-            # actions for design mode
             menu.addAction(configAct)
             menu.addAction(showHostnameAct)
             menu.addAction(changeHostnameAct)
 
-        instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, FW), items)
+        instances = map(lambda item: isinstance(item, ETHSW) or isinstance(item, ATMSW) or isinstance(item, ATMBR) or isinstance(item, FRSW), items)
+        if True in instances:
+            
+            # Action: ChangeHypervisor (Change the hypervisor)
+            changeHypervisor = QtGui.QAction(translate('Scene', 'Set an hypervisor'), menu)
+            changeHypervisor.setIcon(QtGui.QIcon(":/icons/applications.svg"))
+            self.connect(changeHypervisor, QtCore.SIGNAL('triggered()'), self.slotChangeHypervisor)
+            
+            menu.addAction(changeHypervisor)
+            
+        instances = map(lambda item: isinstance(item, ETHSW), items)
+        if True in instances:
+            
+            # Action: MAC Table
+            MACTableAct = QtGui.QAction(translate('Scene', 'MAC Address Table'), menu)
+            MACTableAct.setIcon(QtGui.QIcon(':/icons/inspect.svg'))
+            self.connect(MACTableAct, QtCore.SIGNAL('triggered()'), self.slotMACTable)
+
+            menu.addAction(MACTableAct)
+
+        instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice), items)
         if True in instances:
 
             # Action: Change the console port
@@ -171,27 +188,17 @@ class Scene(QtGui.QGraphicsView):
             suspendAct = QtGui.QAction(translate('Scene', 'Suspend'), menu)
             suspendAct.setIcon(QtGui.QIcon(':/icons/pause.svg'))
             self.connect(suspendAct, QtCore.SIGNAL('triggered()'), self.slotSuspendNode)
+            
+            # Action: Reload (stop and start IOS on hypervisor)
+            reloadAct = QtGui.QAction(translate('Scene', 'Reload'), menu)
+            reloadAct.setIcon(QtGui.QIcon(':/icons/reload.svg'))
+            self.connect(reloadAct, QtCore.SIGNAL('triggered()'), self.slotReloadNode)
 
             menu.addAction(suspendAct)
+            menu.addAction(reloadAct)
             menu.addAction(idlepcAct)
             menu.addAction(StartupConfigAct)
 
-        instances = map(lambda item: isinstance(item, Annotation) or isinstance(item, Pixmap) or isinstance(item, AbstractShapeItem), items)
-        if True in instances:
-        
-            # Action: Lower Z value
-            lowerZvalueAct = QtGui.QAction(translate('Scene', 'Lower one step'), menu)
-            lowerZvalueAct.setIcon(QtGui.QIcon(':/icons/lower_z_value.svg'))
-            self.connect(lowerZvalueAct, QtCore.SIGNAL('triggered()'), self.slotlowerZValue)
-            
-            # Action: Raise Z value
-            raiseZvalueAct = QtGui.QAction(translate('Scene', 'Raise one step'), menu)
-            raiseZvalueAct.setIcon(QtGui.QIcon(':/icons/raise_z_value.svg'))
-            self.connect(raiseZvalueAct, QtCore.SIGNAL('triggered()'), self.slotraiseZValue)
-
-            menu.addAction(lowerZvalueAct)
-            menu.addAction(raiseZvalueAct)
-
         instances = map(lambda item: isinstance(item, Annotation) or isinstance(item, AbstractShapeItem), items)
         if True in instances:
         
@@ -207,6 +214,19 @@ class Scene(QtGui.QGraphicsView):
         deleteAct.setIcon(QtGui.QIcon(':/icons/delete.svg'))
         self.connect(deleteAct, QtCore.SIGNAL('triggered()'), self.slotDeleteNode)
         menu.addAction(deleteAct)
+        
+        # Action: Lower Z value
+        lowerZvalueAct = QtGui.QAction(translate('Scene', 'Lower one step'), menu)
+        lowerZvalueAct.setIcon(QtGui.QIcon(':/icons/lower_z_value.svg'))
+        self.connect(lowerZvalueAct, QtCore.SIGNAL('triggered()'), self.slotlowerZValue)
+            
+        # Action: Raise Z value
+        raiseZvalueAct = QtGui.QAction(translate('Scene', 'Raise one step'), menu)
+        raiseZvalueAct.setIcon(QtGui.QIcon(':/icons/raise_z_value.svg'))
+        self.connect(raiseZvalueAct, QtCore.SIGNAL('triggered()'), self.slotraiseZValue)
+
+        menu.addAction(raiseZvalueAct)
+        menu.addAction(lowerZvalueAct)
 
         menu.exec_(QtGui.QCursor.pos())
 
@@ -218,7 +238,7 @@ class Scene(QtGui.QGraphicsView):
         """ Overloaded function that add the node into the topology
         """
 
-        self.__topology.addNode(node)
+        self.__topology.addNodeFromScene(node)
 
     def calculateIDLEPC(self, router):
         """ Show a splash screen
@@ -231,6 +251,16 @@ class Scene(QtGui.QGraphicsView):
         result = globals.GApp.dynagen.devices[router.hostname].idleprop(lib.IDLEPROPGET)
         return result
 
+    def slotMACTable(self):
+        """ Show Ethernet Switch MAC Address Table
+        """
+
+        for item in self.__topology.selectedItems():
+            if isinstance(item, ETHSW):
+                table = MACTableDialog(item)
+                table.show()
+                table.exec_()
+
     def slotStyle(self):
         """ Change the style of an annotation or a shape item
         """
@@ -253,23 +283,14 @@ class Scene(QtGui.QGraphicsView):
         if style.exec_():
             for item in self.__topology.selectedItems():
                 if isinstance(item, Annotation):
-                    item.setDefaultTextColor(style.color)
-                    item.setFont(style.font)
-                    # reinitialize the rotation
-                    if item.rotation:
-                        item.rotate(-item.rotation)
-                    item.rotation = style.rotation
-                    item.rotate(item.rotation)
+                    command = undo.NewAnnotationStyle(item, style.color, style.font, style.rotation)
+                    item.autoGenerated = False
+                    self.__topology.undoStack.push(command)
                 elif isinstance(item, AbstractShapeItem):
                     pen = QtGui.QPen(style.borderColor, style.borderWidth, style.borderStyle, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
-                    item.setPen(pen)
                     brush = QtGui.QBrush(style.color)
-                    item.setBrush(brush)
-                    # reinitialize the rotation
-                    if item.rotation:
-                        item.rotate(-item.rotation)
-                    item.rotation = style.rotation
-                    item.rotate(item.rotation)
+                    command = undo.NewItemStyle(item, pen, brush, style.rotation)
+                    self.__topology.undoStack.push(command)
 
     def slotIdlepc(self):
         """ Compute an IDLE PC
@@ -325,20 +346,25 @@ class Scene(QtGui.QGraphicsView):
             if ok:
                 selection = str(selection).split(':')[0].strip('* ')
                 index = int(selection)
-                globals.GApp.dynagen.devices[router.hostname].idleprop(lib.IDLEPROPSET, idles[index])
-                QtGui.QMessageBox.information(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),
-                                              unicode(translate("Scene", "Applied idlepc value %s to %s")) % (idles[index], router.hostname))
                 for node in globals.GApp.topology.nodes.values():
                     if isinstance(node, IOSRouter) and node.hostname == router.hostname:
                         dyn_router = node.get_dynagen_device()
                         if globals.GApp.iosimages.has_key(dyn_router.dynamips.host + ':' + dyn_router.image):
                             image = globals.GApp.iosimages[dyn_router.dynamips.host + ':' + dyn_router.image]
-                            debug("Apply IDLE PC " + idles[index] + " for image " + image.filename)
+                            debug("Register IDLE PC " + idles[index] + " for image " + image.filename)
                             image.idlepc =  idles[index]
-                config = router.get_config()
-                config['idlepc'] = idles[index]
-                router.set_config(config)
-                router.setCustomToolTip()
+                            # Apply idle pc to devices with the same IOS image
+                            for device in globals.GApp.topology.nodes.values():
+                                if isinstance(device, IOSRouter) and device.config['image'] == image.filename:
+                                    debug("Apply IDLE PC " + idles[index] + " to " + device.hostname)
+                                    device.get_dynagen_device().idlepc = idles[index]
+                                    config = device.get_config()
+                                    config['idlepc'] = idles[index]
+                                    device.set_config(config)
+                                    device.setCustomToolTip()
+                            break
+                QtGui.QMessageBox.information(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),
+                                              unicode(translate("Scene", "Applied idlepc value %s to %s")) % (idles[index], router.hostname))
         except lib.DynamipsError, msg:
             QtGui.QMessageBox.critical(self, translate("Scene", "Dynamips error"),  unicode(msg))
             return
@@ -362,6 +388,15 @@ class Scene(QtGui.QGraphicsView):
 
         for item in self.__topology.selectedItems():
             item.changeHostname()
+            
+    def slotChangeHypervisor(self):
+        """ Slot called to change hypervisor of selected items
+        """
+
+        for item in self.__topology.selectedItems():
+            if isinstance(item, ETHSW) or isinstance(item, ATMSW) or \
+                isinstance(item, ATMBR) or isinstance(item, FRSW):
+                item.changeHypervisor()
 
     def slotShowHostname(self):
         """ Slot called to show hostnames of selected items
@@ -381,40 +416,45 @@ class Scene(QtGui.QGraphicsView):
         for item in self.__topology.selectedItems():
             if not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem):
                 for link in item.getEdgeList().copy():
-                    if self.__topology.deleteLink(link) == False:
+                    if self.__topology.deleteLinkFromScene(link) == False:
                         if ok_to_delete_node:
                             ok_to_delete_node = False
                         continue
                 if ok_to_delete_node:
-                    self.__topology.deleteNode(item.id)
+                    self.__topology.deleteNodeFromScene(item.id)
             else:
-                self.__topology.removeItem(item)
+                command = undo.DeleteItem(self.__topology, item)
+                self.__topology.undoStack.push(command)
 
     def slotlowerZValue(self):
         """ Lower Z value
         """
-    
+ 
         for item in self.__topology.selectedItems():
-            if isinstance(item, Annotation) or isinstance(item, Pixmap) or isinstance(item, AbstractShapeItem):
-                zvalue = item.zValue()
-                if zvalue > 0:
-                    item.setZValue(zvalue - 1)
-        
+            zvalue = item.zValue()
+            if zvalue > 0:
+                command = undo.NewZValue(item, zvalue - 1)
+                self.__topology.undoStack.push(command)
+            elif isinstance(item, AbstractShapeItem):
+                # shape items can have a z value lower than 0
+                command = undo.NewZValue(item, zvalue - 1)
+                self.__topology.undoStack.push(command)
+
     def slotraiseZValue(self):
         """ Raise Z value
         """
     
         for item in self.__topology.selectedItems():
-            if isinstance(item, Annotation) or isinstance(item, Pixmap) or isinstance(item, AbstractShapeItem):
-                zvalue = item.zValue()
-                item.setZValue(zvalue + 1)
+            zvalue = item.zValue()
+            command = undo.NewZValue(item, zvalue + 1)
+            self.__topology.undoStack.push(command)
 
     def slotConsole(self):
         """ Slot called to launch a console on the selected items
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, FW):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.console()
                 
     def slotChangeConsolePort(self):
@@ -422,7 +462,7 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, FW):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.changeConsolePort()
 
     def slotStartupConfig(self):
@@ -438,7 +478,7 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, FW):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.startNode()
 
     def slotStopNode(self):
@@ -446,7 +486,7 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, FW):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
                 item.stopNode()
 
     def slotSuspendNode(self):
@@ -457,6 +497,14 @@ class Scene(QtGui.QGraphicsView):
             if  isinstance(item, IOSRouter):
                 item.suspendNode()
                 
+    def slotReloadNode(self):
+        """ Slot called to reload the selected items
+        """
+
+        for item in self.__topology.selectedItems():
+            if  isinstance(item, IOSRouter):
+                item.reloadNode()
+                
     def getSourceNode(self):
     
         return globals.GApp.topology.getNode(self.__sourceNodeID)
@@ -481,7 +529,7 @@ class Scene(QtGui.QGraphicsView):
             return
 
         # add the link into the topology
-        if self.__topology.addLink(self.__sourceNodeID, self.__sourceInterface, self.__destNodeID, self.__destInterface) == False:
+        if self.__topology.addLinkFromScene(self.__sourceNodeID, self.__sourceInterface, self.__destNodeID, self.__destInterface) == False:
             self.__isFirstClick = True
         
         self.__sourceNodeID = None
@@ -526,7 +574,7 @@ class Scene(QtGui.QGraphicsView):
         """ Delete an edge from the topology
         """
 
-        self.__topology.deleteLink(edge)
+        self.__topology.deleteLinkFromScene(edge)
 
     def scaleView(self, scale_factor):
         """ Zoom in and out
@@ -582,7 +630,6 @@ class Scene(QtGui.QGraphicsView):
             symbolname = str(event.mimeData().text())
             # Get resource corresponding to node type
             object = None
-            svgrc = ":/icons/default.svg"
             for item in SYMBOLS:
                 if item['name'] == symbolname:
                     renderer_normal = self.renders[symbolname]['normal']
@@ -601,7 +648,7 @@ class Scene(QtGui.QGraphicsView):
             if globals.GApp.workspace.flg_showHostname == True:
                 node.showHostname()
 
-            self.__topology.addNode(node)
+            self.__topology.addNodeFromScene(node)
 
             # Center the node
             pos_x = node.pos().x() - (node.boundingRect().width() / 2)
@@ -625,17 +672,26 @@ class Scene(QtGui.QGraphicsView):
             item_type = type(item)
             if item_type == Ethernet or item_type == Serial:
                 show = False
-        if show and event.button() == QtCore.Qt.RightButton and not globals.addingLinkFlag:
+                
+        if show and event.modifiers() & QtCore.Qt.ShiftModifier and event.button() == QtCore.Qt.LeftButton and item and not globals.addingLinkFlag:
+            item.setSelected(True)
+        elif show and event.button() == QtCore.Qt.RightButton and not globals.addingLinkFlag:
             if item:
+                if not event.modifiers() & QtCore.Qt.ShiftModifier:
+                    for it in globals.GApp.topology.items():
+                        it.setSelected(False)
                 item.setSelected(True)
-            self.showContextualMenu()
+                self.showContextualMenu()
+            elif len(self.__topology.selectedItems()) > 1:
+                self.showContextualMenu()
         elif event.button() == QtCore.Qt.LeftButton and globals.addingNote:
             note = Annotation()
             note.setPos(self.mapToScene(event.pos()))
             pos_x = note.pos().x()
             pos_y = note.pos().y() - (note.boundingRect().height() / 2)
-            note.setPos(pos_x, pos_y)
-            globals.GApp.topology.addItem(note)
+            note.setPos(pos_x, pos_y)      
+            command = undo.AddItem(self.__topology, note, translate("Scene", "annotation"))
+            self.__topology.undoStack.push(command)
             note.editText()
             globals.GApp.workspace.action_AddNote.setChecked(False)
             globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
@@ -643,14 +699,16 @@ class Scene(QtGui.QGraphicsView):
         elif event.button() == QtCore.Qt.LeftButton and globals.drawingRectangle:
             size = QtCore.QSizeF(200, 100)
             item = Rectangle(self.mapToScene(event.pos()),  size)
-            globals.GApp.topology.addItem(item)
+            command = undo.AddItem(self.__topology, item, translate("Scene", "rectangle"))
+            self.__topology.undoStack.push(command)
             globals.GApp.workspace.action_DrawRectangle.setChecked(False)
             globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
             globals.drawingRectangle = False
         elif event.button() == QtCore.Qt.LeftButton and globals.drawingEllipse:
             size = QtCore.QSizeF(200, 200)
             item = Ellipse(self.mapToScene(event.pos()),  size)
-            globals.GApp.topology.addItem(item)
+            command = undo.AddItem(self.__topology, item, translate("Scene", "ellipse"))
+            self.__topology.undoStack.push(command)
             globals.GApp.workspace.action_DrawEllipse.setChecked(False)
             globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
             globals.drawingEllipse = False
@@ -659,20 +717,15 @@ class Scene(QtGui.QGraphicsView):
 
     def mouseDoubleClickEvent(self, event):
 
-        if not globals.addingLinkFlag:
-            item = self.itemAt(event.pos())
-            if isinstance(item, Annotation) or isinstance(item, Pixmap):
-                QtGui.QGraphicsView.mouseDoubleClickEvent(self, event)
-            elif item and not isinstance(item, AbstractEdge):
-                item.setSelected(True)
-                self.slotConfigNode()
+        item = self.itemAt(event.pos())
+        if not globals.addingLinkFlag and isinstance(item, AbstractNode):
+            item.setSelected(True)
+            self.slotConfigNode()
         else:
             QtGui.QGraphicsView.mouseDoubleClickEvent(self, event)
 
     def mouseMoveEvent(self, event):
 
-        # update new edge position
-#        globals.GApp.workspace.statusbar.showMessage("%d, %d" % (event.pos().x(), event.pos().y()))
         if (self.newedge):
             self.newedge.setMousePoint(self.mapToScene(event.pos()))
             event.ignore()
