@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007 GNS-3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 import GNS3.Globals as globals
@@ -143,7 +143,7 @@ class Page_FRSW(QtGui.QWidget, Ui_FRSWPage):
         if config:
             FRSWconfig = config
         else:
-            FRSWconfig  = self.node.config
+            FRSWconfig  = self.node.duplicate_config()
 
         FRSWconfig['mapping'] = self.mapping
         FRSWconfig['ports'] = []
